Index: contrib/scripts/astcli
===================================================================
--- contrib/scripts/astcli	(revision 114091)
+++ contrib/scripts/astcli	(working copy)
@@ -12,6 +12,7 @@
 #
 my ($user, $pw, $host, $port, $interactive, $save) = (undef, undef, 'localhost', 5038, 0, 0);
 my $EOL = "\r\n"; # Standard End of Line
+my @commands;
 process_credentials('/etc/astcli.conf');
 process_credentials("$ENV{HOME}/.astcli") if defined $ENV{HOME};
 GetOptions("username=s" => \$user, "secret=s" => \$pw, "host=s" => \$host, "port=s" => \$port, "readline" => \$interactive, "write" => \$save);
@@ -56,7 +57,7 @@
 	}
 	$response =~ s/Privilege: Command$EOL//;
 	$response =~ s/Response: Follows$EOL//;
-	print $response;
+	return $response;
 }
 
 sub login {
@@ -90,30 +91,27 @@
 		print STDERR "Falling back to standard mode, Unable to load Term::Readline for readline mode\n" unless $interactive;
 	}
 	if ($interactive) {
+		load_commands();
 		my $term = new Term::ReadLine 'Command Line Interface';
 		my $prompt = "$host*CLI> ";
 		my $attribs = $term->Attribs;
-		$attribs->{completion_function} = sub {
-			my ($text, $line, $start) = @_;
-			# Stub function for tab auto completion for those feeling adventurous
-			return;
-		};
+		$attribs->{completion_function} = \&tab_completion;
 		while (defined($_ = $term->readline($prompt))) {
 			(logoff() and exit) if $_ =~ /exit|quit/; # Give them a way to exit the "terminal"
-			send_command($_);
+			print send_command($_);
 		}	
 	} else {
 		while (<>) {
 			chomp;
 			(logoff() and exit) if $_ =~ /exit|quit/; # If someone accidentally ends up here, let them exit
-			send_command($_);
+			print send_command($_);
 		}
 	}
 	exit 0;
 }
 
 # Otherwise just send the command:
-send_command($action);
+print send_command($action);
 
 # parses a configuration file into the global $user and $pw.
 sub process_credentials {
@@ -144,3 +142,25 @@
 	exit;
 }
 
+sub load_commands {
+    # Load the listing of available commands into the @commands array via the help command
+    my %command_line = map { m/^\s+(.*?)\s{2}.*$/ => undef} grep {/^\s+.*?\s{2}.*$/} split (/\r*\n/, send_command('help'));
+    @commands = sort keys %command_line;
+}
+
+sub tab_completion {
+    my ($word, $buffer, $offset) = @_;
+    # Find the number of words on the command line
+    my $wc = scalar(my @words = (split(/\s+/, $buffer)));
+    # Minus one word for the array offset unless the current word is blank or we're on the 1st word
+    $wc-- unless $wc == 0 || $word eq '';
+    my %command;
+    # Get a listing of commands that match the full command line.
+    foreach (grep { /^$buffer/; } @commands){
+	# Split those commands on the space.
+	my @line = split(/\s+/, $_);
+	# Build a hash of the next possible words
+	$command{$line[$wc]} = undef if defined $line[$wc] and $line[$wc] ne $word;
+    }
+    return sort keys %command;
+}
\ No newline at end of file
